@@ -1,5 +1,6 @@
 import ibm2ieee
 import numpy as np
+from numpy.lib import recfunctions as rfn
 from typing import Literal
 from pathlib import Path
 from enum import Enum
@@ -10,6 +11,29 @@ import functools
 from rich.progress import track, Progress
 
 
+def iter_chunks_slice(itrace, n, chunk_size, progressbar=''):
+    a, b, st = (slice(None) if itrace is Ellipsis else itrace).indices(n)
+    total_len = max(0, (b - a + (st - 1)) // st if st > 0 else (a - b + (-st - 1)) // (-st))
+    cnt_chunk = (total_len + chunk_size - 1) // chunk_size
+    chunk_iters = track(range(cnt_chunk), description=progressbar, refresh_per_second=1, update_period=2) if progressbar else range(cnt_chunk)
+    for i in chunk_iters:
+        start = a + i * chunk_size * st
+        end   = a + min((i+1) * chunk_size, total_len) * st
+        # 计算 slice 的真实长度
+        if (st > 0 and start >= b) or (st < 0 and start <= b):
+            continue
+        yield slice(start, end, st)
+
+def iter_chunks_idxs(itrace, chunk_size, progressbar=''):
+    idxs = [itrace] if np.isscalar(itrace) else itrace
+    cnt_chunk = (len(idxs) + chunk_size - 1) // chunk_size
+    chunk_iters = track(range(cnt_chunk), description=progressbar, refresh_per_second=1, update_period=2) if progressbar else range(cnt_chunk)
+    for i in chunk_iters:
+        chunk = idxs[i*chunk_size:(i+1)*chunk_size]
+        if len(chunk) == 0:
+            continue
+        yield chunk
+
 def parallelize_io(min_items_for_parallel=100000):
     """
     一个装饰器，用于将针对长序列的IO操作并行化。
@@ -304,6 +328,7 @@ class SEGYKey(object):
 
 class SEGYMap(CVDFile, SEGYKey):
     """基于numpy.memmap的SEG-Y文件读取类"""
+    chunk_size = 1000 # 读取道头时，每次读取的道数
 
     def __enter__(self):
         return self
@@ -322,7 +347,8 @@ class SEGYMap(CVDFile, SEGYKey):
         self._mode: _MemMapModeKind = mode
         self._cache_text_header = None
         self._cache_binary_header = None
-        self._trace = None  # 句柄，用于读取和写入道数据
+        self._trace_memmap: functools.partial = None  # 用于创建道数据内存映射
+        self._trace : np.memmap = None
         # 关键信息获取,建立与道的映射关系
         self.ReadKeyInfo()
         self.SetTraceMapping()
@@ -420,7 +446,7 @@ class SEGYMap(CVDFile, SEGYKey):
             raise IOError(f'the size of value must be {self.config.binary_header_size}!')
 
     @parallelize_io(min_items_for_parallel=100000)
-    def traceData(self, itrace, item=None):
+    def traceData(self, itrace, item=None, progressbar=''):
         """
         用接口来读取对应地震数据
         :param itrace:  地震数据读取索引
@@ -456,39 +482,69 @@ class SEGYMap(CVDFile, SEGYKey):
         self._trace.flush()
         return True
 
-    def traceHeader(self, itrace, *keys: TraceField):
+    def traceHeader(self, itrace, *keys: TraceField, progressbar=''):
         """
 
-        :param itrace:   地震数据读取索引
+        :param itrace:   地震数据读取索引: 数组、切片、整数、Ellipsis
         :param keys:  枚举类型的关键字
+        :param progressbar:  进度条描述, 空字符串不显示进度条
         :return:
         """
-        output, ks = lambda ks: self._trace[itrace][Trace.Header.name][ks], ...
-        if len(keys) == 0:
-            if np.isscalar(itrace):
-                return self._trace[itrace][Trace.Header.name]
-            return np.array(self._trace[itrace][Trace.Header.name])
-        if len(keys) == 1:
-            ks = keys[0].name
-        if len(keys) > 1:
-            ks = [k.name for k in keys]
-        if np.isscalar(itrace):
-            return output(ks)
-        return np.array(output(ks))
 
-    def SetTraceHeader(self, itrace, traceHeader, *keys: TraceField):
+        if itrace is Ellipsis or isinstance(itrace, slice): # 切片
+            iter_chunks = iter_chunks_slice(itrace, self.trace_cnt, self.chunk_size, progressbar=progressbar)
+            a, b, st  = (slice(None) if itrace is Ellipsis else itrace).indices(self.trace_cnt)
+            size = max(0, (b - a + (st - 1)) // st if st > 0 else (a - b + (-st - 1)) // (-st))
+        else: # 整数或数组
+            iter_chunks = iter_chunks_idxs(itrace, self.chunk_size, progressbar=progressbar)
+            size = len([itrace] if np.isscalar(itrace) else itrace)
+        fields = [k.name for k in keys]
+        output = np.empty(size, dtype=self.config.trace_header_dtype[fields])
+        offset = 0
+        for chunk in iter_chunks:
+            view  = self._trace[chunk][Trace.Header.name]
+            chunk_len = len(view)
+            if chunk_len == 0:
+                continue
+            np.copyto(output[offset:offset+chunk_len][fields], view[fields], casting='no')
+            offset += chunk_len
+      
+        return output[0][0] if output.size == 1 and len(fields) == 1 else output
+
+    def SetTraceHeader(self, itrace, traceHeader, progressbar=''):
         """
         :param itrace:
-        :param keys:
+        :param traceHeader:
+        :param progressbar:  进度条描述, 空字符串不显示进度条
         :return:
         """
-        if len(keys) == 0:
-            for idx, data in zip(itrace, traceHeader):
-                self._trace[idx][Trace.Header.name] = data
-        else:
-            ks = keys[0].name if len(keys) == 1 else [k.name for k in keys]
-            for idx, data in zip(itrace, traceHeader):
-                self._trace[idx][Trace.Header.name][ks] = data
+        if itrace is Ellipsis or isinstance(itrace, slice): # 切片
+            iter_chunks = iter_chunks_slice(itrace, self.trace_cnt, self.chunk_size, progressbar=progressbar)
+            a, b, st  = (slice(None) if itrace is Ellipsis else itrace).indices(self.trace_cnt)
+            size = max(0, (b - a + (st - 1)) // st if st > 0 else (a - b + (-st - 1)) // (-st))
+        else: # 整数或数组
+            iter_chunks = iter_chunks_idxs(itrace, self.chunk_size, progressbar=progressbar)
+            size = len([itrace] if np.isscalar(itrace) else itrace)
+
+        # 按 memmap 内部顺序获取字段
+        aligned_fields = [f for f in self.config.trace_header_dtype.names if f in traceHeader.dtype.names]
+
+        if size != len(traceHeader):
+            raise ValueError(f"the size of traceHeader must be {size}!")
+
+        # 对 traceHeader 生成按 memmap 顺序排列的视图，不复制数据
+        traceHeader_view = rfn.repack_fields(traceHeader[aligned_fields])
+
+        # 直接赋值到 memmap
+        offset = 0
+        for chunk in iter_chunks:
+            view = self._trace[chunk][Trace.Header.name]
+            chunk_len = len(view)
+            if chunk_len == 0:
+                continue
+            view[aligned_fields] = traceHeader_view[offset:offset+chunk_len]
+            offset += chunk_len
+
         self._trace.flush()
 
     def SetTraceMapping(self, set_trace_cnt: int = 1):
@@ -507,7 +563,8 @@ class SEGYMap(CVDFile, SEGYKey):
         file_trace_cnt = (self._file_path.stat().st_size - (
                     self.config.text_header_size + self.config.binary_header_size)) // trace_size
         self.trace_cnt = max(file_trace_cnt, set_trace_cnt)     #  ①创建segy时默认道数为1，读取segy会自动读取对应道数
-        self._trace = np.memmap(self._file_path, dtype=dtype, mode=self.mode, offset=offset, shape=(self.trace_cnt,))
+        self._trace_memmap = functools.partial(np.memmap, filename=self._file_path, dtype=dtype, mode=self.mode, offset=offset, shape=(self.trace_cnt,))
+        self._trace = self._trace_memmap()
         self.coord_scale = self.traceHeader(0, TraceField.CoordScale)
         # self.coord_scale = 1 if not self.coord_scale else pow(abs(self.coord_scale) + 0.0,  1 if self.coord_scale > 0 else -1)
         self.coord_scale = 1 if not self.coord_scale else abs(self.coord_scale)
@@ -580,4 +637,13 @@ class SEGYMap(CVDFile, SEGYKey):
     def __len__(self):
         """返回道数"""
         return self.trace_cnt
+    
 
+    def __getstate__(self) -> object:
+        state = super().__getstate__()
+        state['_trace'] = None
+        return state
+    
+    def __setstate__(self, state: object) -> None:
+        self.__dict__.update(state)
+        self._trace = self._trace_memmap()
