@@ -146,7 +146,9 @@ class SeisData(SeisGrid, SEGYMap):
         
         # Case 1: 切片范围的start和stop都不是Horiz对象（即常规的时间/深度切片）
         if hasCls(Horiz, arg.start, arg.stop) == 0:  # 都为Horiz
-            data, iTraces = np.empty((*shape, self.smp_seq[arg].size), dtype='f4'), iTraces[state]
+            # 后续单道数据处理，都会在self.traceData中嵌套处理函数，所以无法确定使用self.traceData后， k轴有多少数据了
+            one_trace = self.traceData(iTraces if np.isscalar(iTraces) else iTraces[:2], [..., arg])
+            data, iTraces = np.empty((*shape, one_trace.shape[-1]), dtype='f4'), iTraces[state]
             data[state], data[~state] = self.traceData(iTraces, [..., arg]), self.vNAN
             # data[state], data[~state] = self._trace[iTraces][Trace.Data.name][..., arg], self.vNAN
             return data
@@ -220,12 +222,10 @@ class SeisData(SeisGrid, SEGYMap):
         # 建立
         # self.SetTraceMapping()
         DType = dtype({'names': [self.kINLINE, self.kXLINE, self.kX, self.kY, self.kITRACE],
-                       'formats': ['i4', 'i4', 'i4', 'f4', 'i4']})
+                       'formats': ['i4', 'i4', 'i4', 'i4', 'i4']})
         dataRows = self.traceHeader(..., TraceField.InlineID, TraceField.XlineID, TraceField.CoordX, TraceField.CoordY,
-                                    TraceField.Index).astype(DType)
-        # dataRows = np.array(self._trace[Trace.Header.name][[
-        #     TraceField.InlineID.name, TraceField.XlineID.name, TraceField.CoordX.name,
-        #     TraceField.CoordY.name, TraceField.Index.name]], dtype=DType)
+                                    TraceField.Index, progressbar=f'    loading{self._file_path.name}:')
+        dataRows = dataRows.astype(DType)
         dataRows[self.kX] = dataRows[self.kX] / self.coord_scale
         dataRows[self.kY] = dataRows[self.kY] / self.coord_scale
         dataRows[self.kITRACE] = np.arange(self.trace_cnt)
